#include "driverC350.h"
#include "Font.h"
unsigned char Font;

drvC350::drvC350()
{
  ColorPixel = 0xff;
  ColorFill = 0;
  spi = SPI();
}
void drvC350::Init(void)
{
  //настройка портов
  pinMode(DataCmd, OUTPUT);
  pinMode(Reset, OUTPUT);
  pinMode(LedDriver, OUTPUT);
  digitalWrite(DataCmd, HIGH);
  digitalWrite(Reset, HIGH);
  digitalWrite(LedDriver, HIGH);
  //LcdDdr |= (1<<ChipEnable)|(1<<DataCmd)|(1<<Reset)|(1<<Clk)|(1<<Mosi)|(1<<LedDriver);
  //LcdPort |= (1<<ChipEnable)|(1<<DataCmd)|(1<<Reset)|(1<<Clk)|(1<<Mosi)|(1<<LedDriver);
 
  //сигнал сброса для жкд
  digitalWrite(Reset, LOW);
//  CBR(LcdPort, Reset);
  delay(12);
  digitalWrite(Reset, HIGH);
//  SBR(LcdPort, Reset);
  
  //настройка spi
  //SPCR = (0<<SPIE)|(1<<SPE)|(0<<DORD)|(1<<MSTR)|(0<<CPOL)|(1<<SPR1)|(1<<SPR0);
  //SBR(SPSR, SPI2X);
}

//посылает команду по SPI на жкд
void drvC350::SendCom(byte data)
{
  Disable_SPI_Interrupt;
  spi.ChipSelect();
//  CBR(LcdPort,ChipEnable);
  digitalWrite(DataCmd, LOW);
  spi.transfer(data);
//  CBR(LcdPort, DataCmd);
//  SPDR = data;
//  while(SBRC(SPSR, SPIF));    
//  SBR(LcdPort, ChipEnable);
  spi.ChipUnSelect();
  digitalWrite(DataCmd, HIGH);
  Enable_SPI_Interrupt;
}

//посылает данные по SPI на жкд
void drvC350::SendData(byte data)
{
    Disable_SPI_Interrupt;
    spi.ChipSelect();
//  CBR(LcdPort,ChipEnable);
    digitalWrite(DataCmd, HIGH);
    spi.transfer(data);
//  SBR(LcdPort, DataCmd);
//  SPDR = data;
//  while(SBRC(SPSR, SPIF));    
//  SBR(LcdPort, ChipEnable);
    spi.ChipUnSelect();
    digitalWrite(DataCmd, HIGH);
    Enable_SPI_Interrupt;
}

void drvC350::DisableIntOsc(){  SendCom(ComDisableIntOsc);}
void drvC350::EnableIntOsc() {  SendCom(ComEnableIntOsc); }
void drvC350::ExitSleepMode() {SendCom(ComExitSleepMode);}
void drvC350::OnDisplay() {SendCom(ComOnDisplay);}
void drvC350::LedDriverOn() {digitalWrite(LedDriver, HIGH);} //это для подсветки, нужно постоянно запускать чтобы удвоитель напряжения работал

//рисует линию на экране
void drvC350::DrawLine(byte x1, byte y1, byte x2, byte y2, byte redgreen, byte blue)
{
  SendCom(ComDrawLine);
  SendData(x1);
  SendData(y1);  
  SendData(x2);  
  SendData(y2);  
  SendData(redgreen);
  SendData(blue);  
}

//рисует прямоугольник на экране, первый цвет - цвет рамки, второй - цвет заполнения
//заполнение должно быть разрешено специальной командой, по умолчанию оно выключено
void drvC350::DrawRectangle(byte x1, byte y1, byte x2, byte y2, byte redgreen1, byte blue1, byte redgreen2, byte blue2)
{
  SendCom(ComDrawRectangle);
  SendData(x1);
  SendData(y1);  
  SendData(x2);  
  SendData(y2);  
  SendData(redgreen1);
  SendData(blue1);  
  SendData(redgreen2);
  SendData(blue2); 
}

//очищает область дисплея, заданную координатами
void drvC350::ClearDisplay(byte x1, byte y1, byte x2, byte y2)
{
  SendCom(ComClearWindow);
  SendData(x1);
  SendData(y1);  
  SendData(x2);  
  SendData(y2);
}

//разрешает заполнение
void drvC350::FillEnable(void)
{
  SendCom(ComFillEnableDisable);
  SendData(1);
}

//запрещает заполнение
void drvC350::FillDisable(void)
{
  SendCom(ComFillEnableDisable);
  SendData(0);
}

//эту функцию частично передрал из описания на контроллер
//не вдаваясь в подробности
//тут полно настроек контроллера экрана
void drvC350::SetupDisplay(void)
{
    SendCom(ComSetPowerControlReg );
    SendData(0x0b);
    
    SendCom(ComSetDisplayControl);
    SendData(0);     
    SendData(16);
    SendData(0);  
    
    SendCom(ComSelectPwmFrc);
    SendData(0x28);   
    SendData(0x2c);  
    SendData(0x05); 
    
    SendCom(ComSetBiasingRatio);
    SendData(0x3); 
      
    SendCom(ComSetComOutputScanDir);
    SendData(0x1); 
}



//устанавливает контраст
//если его неправильно выставить, то на экране нельзя будет ничего увидеть
void drvC350::SetContrastIntReg(byte contrast, byte regulator)
{
  SendCom(ComSetContrastIntReg);
  SendData(contrast);
  SendData(regulator);  
}



//устанавливает таблицу перекодировки
//усли ее не установить, то данные записываемые на экран
//командой write не будут отображаться
void drvC350::SetLookUpTable(void)
{
  SendCom(ComSetLookUpTable);
  //red
  SendData(0x0);
  SendData(0x2);  
  SendData(0x4); 
  SendData(0x6);  
  SendData(0x8);
  SendData(0xa);  
  SendData(0xc);
  SendData(0xe);    
  //green
  SendData(0x0);
  SendData(0x2);  
  SendData(0x4); 
  SendData(0x6);  
  SendData(0x8);
  SendData(0xa);  
  SendData(0xc);
  SendData(0xe); 
  //blue
  SendData(0x3);
  SendData(0x7);  
  SendData(0xb);
  SendData(0xf); 
}

void drvC350::SetDataOutputScanDir(byte adrscan, byte rgbcol, byte grayscale)
{
  SendCom(ComSetDataOutputScanDir);
  SendData(adrscan&0x7);
  SendData(rgbcol&0x7); 
  SendData(grayscale&0x3);  
}
//________________________знакогенератор________________________________________

//устанавливает цвет шрифта и цвет заполнения вокруг
void drvC350::SetColor(byte cp, byte cf)
{
  ColorPixel = cp;
  ColorFill = cf;  
}

//отображает на экране символ
void drvC350::SendLcdSymbol(byte data, byte ColAdr, byte PageAdr)
{
  unsigned char PointerFont = data<<1; 
  //byte tmp = 0;
  
  SendCom(ComSetColumnAddress);
  SendData(ColAdr);
  SendData(ColAdr+4); 
  SendCom(ComSetPageAddress);
  SendData(PageAdr);  
  SendData(PageAdr+7);
  SendCom(ComWriteDisplayData);    
  for(byte i = 0; i<5; i++){
      byte fontdata = pgm_read_byte(Font + PointerFont);  
      //byte fontdata = Font;
	  //byte fontdata = 0x20;
      for(byte j = 0; j<8; j++){
        if (bitRead(fontdata, j)!=0) SendData(ColorPixel); // pixel ON
        else SendData(ColorFill);  // pixel OFF
      }
      PointerFont++;
  }
}

//отображает на экране строку
void drvC350::SendLcdString(char* String, byte ColAdr, byte PageAdr)
{
  unsigned char Data = 0;
  while(*String){
    Data = *String++;
    SendLcdSymbol(Data, ColAdr,PageAdr);
    ColAdr+=6;
  }
}

//___________________дополнительные графические функции_________________________

//отображает на экране один пиксел
void drvC350::PutPixel(byte ColAdr, byte PageAdr)
{
   SendCom(ComSetColumnAddress);
   SendData(ColAdr);
   SendData(ColAdr); 
   SendCom(ComSetPageAddress);
   SendData(PageAdr);  
   SendData(PageAdr);
   SendCom(ComWriteDisplayData);
   SendData(ColorPixel);
}

//рисут на экране окружность по алгоритму Брезенхема
void drvC350::DrawCircle(byte xc,byte yc,byte r,byte color)
{
  int  x,y,d;
  byte tmp = ColorPixel;
  ColorPixel = color;
  y = r; 
  d = 3-2*r;
  x = 0;
  while(x <= y)
  {
  PutPixel(x+xc,y+yc);
  PutPixel(x+xc,-y+yc);
  PutPixel(-x+xc,-y+yc);
  PutPixel(-x+xc,y+yc);
  PutPixel(y+xc,x+yc);
  PutPixel(y+xc,-x+yc);
  PutPixel(-y+xc,-x+yc);
  PutPixel(-y+xc,x+yc);
  if (d<0) {
    d = d+4*x+6;
  } else {
   d = d+4*(x-y)+10;
   y--;
  }
  x++;
  };  
  ColorPixel = tmp; 
}

void drvC350::SetAreaScroll(byte TopBlockAddress, byte BottomBlockAddress, byte NumberBlocks, byte ScrollMode)
{
   SendCom(ComSetAreaScroll);
   SendData(TopBlockAddress);
   SendData(BottomBlockAddress); 
   SendData(NumberBlocks); 
   SendData(ScrollMode); 
 }

void drvC350::SetScrollStart(byte StartBlockAddress)
{
   SendCom(ComSetScrollStart);
   SendData(StartBlockAddress);
}

void drvC350::DisplayOnOff(byte OnOff)
{
  if(OnOff==0)   {
    digitalWrite(LedDriver, LOW);
    SendCom(ComDisplayOff);  
  }
  else  {
    digitalWrite(LedDriver, HIGH);
      SendCom(ComDisplayOn);  
    }
}
